@@ -53,7 +53,11 @@ where
     Self: Sized,
     ServerInfo: Clone + Serialize,
 {
-    fn get_render_context(&self, request_route: Option<Route>) -> RenderContext<ServerInfo, Self>;
+    fn render_context<QueryParameters: Clone + Serialize>(
+        &self,
+        request_route: Option<Route>,
+        query_parameters: QueryParameters,
+    ) -> RenderContext<ServerInfo, QueryParameters, Self>;
 
     fn new_template(
         &self,
@@ -369,15 +373,22 @@ impl<'engine, ServerInfo> ContentEngine<ServerInfo>
 where
     ServerInfo: Clone + Serialize,
 {
-    fn get_render_context(&self, request_route: Option<Route>) -> RenderContext<ServerInfo, Self> {
+    fn render_context<QueryParameters: Clone + Serialize>(
+        &self,
+        route: Option<Route>,
+        query_parameters: QueryParameters,
+    ) -> RenderContext<ServerInfo, QueryParameters, Self> {
         RenderContext {
             content_engine: self,
             data: RenderData {
                 server_info: self.server_info.clone(),
                 index: self.index.clone(),
-                request_route,
                 target_media_type: None,
                 error_code: None,
+                request: RequestData {
+                    route,
+                    query_parameters,
+                },
             },
         }
     }
@@ -415,6 +426,7 @@ mod tests {
     use super::*;
     use crate::test_lib::*;
     use ::mime;
+    use serde_json::json;
     use test_env_log::test;
 
     type TestContentEngine<'a> = FilesystemBasedContentEngine<'a, ()>;
@@ -458,7 +470,7 @@ mod tests {
                 )
                 .expect("Template could not be parsed");
             let rendered = renderable
-                .render(content_engine.get_render_context(None), &[mime::TEXT_HTML])
+                .render(content_engine.render_context(None, ()), &[mime::TEXT_HTML])
                 .expect(&format!("Template rendering failed for `{}`", template));
             let actual_output = media_to_string(rendered);
 
@@ -505,7 +517,7 @@ mod tests {
 
         let template = "this is partial: {{> abc.html.hbs}}";
         let expected_output =
-            "this is partial: a\nb\n\nc\n\nsubdirectory entries:\n/subdirectory/c";
+            "this is partial: a\nb\n\nc\n\nsubdirectory entries:\n/subdirectory/c\n";
 
         let renderable = content_engine
             .new_template(
@@ -514,7 +526,7 @@ mod tests {
             )
             .expect("Template could not be parsed");
         let rendered = renderable
-            .render(content_engine.get_render_context(None), &[mime::TEXT_HTML])
+            .render(content_engine.render_context(None, ()), &[mime::TEXT_HTML])
             .expect(&format!("Template rendering failed for `{}`", template));
         let actual_output = media_to_string(rendered);
 
@@ -536,13 +548,13 @@ mod tests {
         let content_engine = shared_content_engine.read().unwrap();
 
         let route = route("/abc");
-        let expected_output = "a\nb\n\nc\n\nsubdirectory entries:\n/subdirectory/c";
+        let expected_output = "a\nb\n\nc\n\nsubdirectory entries:\n/subdirectory/c\n";
 
         let content = content_engine
             .get(&route)
             .expect("Content could not be found");
         let rendered = content
-            .render(content_engine.get_render_context(None), &[mime::TEXT_HTML])
+            .render(content_engine.render_context(None, ()), &[mime::TEXT_HTML])
             .expect(&format!(
                 "Template rendering failed for content at '{}'",
                 route
@@ -607,7 +619,7 @@ mod tests {
             )
             .expect("Template could not be parsed");
         let rendered = renderable
-            .render(content_engine.get_render_context(None), &[mime::TEXT_HTML])
+            .render(content_engine.render_context(None, ()), &[mime::TEXT_HTML])
             .expect(&format!("Template rendering failed for `{}`", template));
         let actual_output = media_to_string(rendered);
 
@@ -644,7 +656,7 @@ mod tests {
                 )
                 .expect("Template could not be parsed");
             let result =
-                renderable.render(content_engine.get_render_context(None), &[mime::TEXT_HTML]);
+                renderable.render(content_engine.render_context(None, ()), &[mime::TEXT_HTML]);
             assert!(
                 result.is_err(),
                 "Content was successfully rendered for invalid template `{}`, but it should have failed",
@@ -671,7 +683,7 @@ mod tests {
                 None => panic!("No content was found at '{}'", route),
                 Some(renderable) => {
                     let result = renderable
-                        .render(content_engine.get_render_context(None), &[mime::TEXT_HTML]);
+                        .render(content_engine.render_context(None, ()), &[mime::TEXT_HTML]);
                     assert!(
                         result.is_err(),
                         "Content was successfully rendered for `{}`, but this should have failed \
@@ -698,7 +710,7 @@ mod tests {
                 MediaType::from_media_range(mime::TEXT_HTML).unwrap(),
             )
             .expect("Template could not be created");
-        let result = template.render(content_engine.get_render_context(None), &[mime::TEXT_PLAIN]);
+        let result = template.render(content_engine.render_context(None, ()), &[mime::TEXT_PLAIN]);
 
         assert!(
             result.is_err(),
@@ -724,7 +736,7 @@ mod tests {
                 None => panic!("No content was found at '{}'", route),
                 Some(renderable) => {
                     let result = renderable.render(
-                        content_engine.get_render_context(None),
+                        content_engine.render_context(None, ()),
                         &[target_media_type],
                     );
                     assert!(
@@ -756,7 +768,7 @@ mod tests {
                             MediaType::from_media_range(mime::TEXT_PLAIN).unwrap(),
                         )
                         .expect("Test template was invalid")
-                        .render(content_engine.get_render_context(None), &[mime::TEXT_PLAIN])
+                        .render(content_engine.render_context(None, ()), &[mime::TEXT_PLAIN])
                         .expect("Failed to render unregistered template"),
                 ),
                 mime::TEXT_PLAIN.essence_str(),
@@ -766,7 +778,7 @@ mod tests {
                     content_engine
                         .get(&route("/echo-target-media-type"))
                         .expect("Test template does not exist")
-                        .render(content_engine.get_render_context(None), &[mime::TEXT_HTML])
+                        .render(content_engine.render_context(None, ()), &[mime::TEXT_HTML])
                         .expect("Failed to render registered template"),
                 ),
                 mime::TEXT_HTML.essence_str(),
@@ -796,7 +808,7 @@ mod tests {
             .get(&route)
             .expect("Content could not be found");
         let rendered = content
-            .render(content_engine.get_render_context(None), &[mime::TEXT_PLAIN])
+            .render(content_engine.render_context(None, ()), &[mime::TEXT_PLAIN])
             .expect(&format!("Rendering failed for content at '{}'", route));
         let actual_output = media_to_string(rendered);
 
@@ -824,7 +836,7 @@ mod tests {
             .get(&route1)
             .expect("Content could not be found");
         let rendered = content
-            .render(content_engine.get_render_context(None), &[mime::TEXT_PLAIN])
+            .render(content_engine.render_context(None, ()), &[mime::TEXT_PLAIN])
             .expect(&format!("Rendering failed for content at '{}'", route1));
         let actual_output = media_to_string(rendered);
 
@@ -844,7 +856,7 @@ mod tests {
             .get(&route2)
             .expect("Content could not be found");
         let rendered = content
-            .render(content_engine.get_render_context(None), &[mime::TEXT_PLAIN])
+            .render(content_engine.render_context(None, ()), &[mime::TEXT_PLAIN])
             .expect(&format!("Rendering failed for content at '{}'", route2));
         let actual_output = media_to_string(rendered);
 
@@ -870,14 +882,14 @@ mod tests {
             .get(&route)
             .expect("Content could not be found");
 
-        let result1 = content.render(content_engine.get_render_context(None), &[mime::TEXT_PLAIN]); // Not text/html!
+        let result1 = content.render(content_engine.render_context(None, ()), &[mime::TEXT_PLAIN]); // Not text/html!
         assert!(
             result1.is_err(),
             "Rendering content at '{}' succeeded when it should have failed",
             route,
         );
 
-        let result2 = content.render(content_engine.get_render_context(None), &[mime::TEXT_HTML]);
+        let result2 = content.render(content_engine.render_context(None, ()), &[mime::TEXT_HTML]);
         assert!(
             result2.is_ok(),
             "Rendering content at '{}' failed when it should have succeeded",
@@ -899,7 +911,7 @@ mod tests {
 
         let media = content
             .render(
-                content_engine.get_render_context(None),
+                content_engine.render_context(None, ()),
                 &[mime::APPLICATION_OCTET_STREAM],
             )
             .expect(&format!(
@@ -921,7 +933,7 @@ mod tests {
             .expect("Content engine could not be created");
         let content_engine = shared_content_engine.read().unwrap();
 
-        let route = route("/template");
+        let route = route("/get-pwd");
         let expected_output = format!(
             "this is pwd from subdirectory:\n{}/samples/executables/subdirectory\n",
             PROJECT_DIRECTORY
@@ -931,7 +943,7 @@ mod tests {
             .get(&route)
             .expect("Content could not be found");
         let rendered = content
-            .render(content_engine.get_render_context(None), &[mime::TEXT_PLAIN])
+            .render(content_engine.render_context(None, ()), &[mime::TEXT_PLAIN])
             .expect(&format!("Rendering failed for content at '{}'", route));
         let actual_output = media_to_string(rendered);
 
@@ -975,4 +987,46 @@ mod tests {
             );
         }
     }
+
+    #[test]
+    fn templates_receive_query_parameters() {
+        let shared_content_engine = TestContentEngine::from_content_directory(
+            arbitrary_content_directory_with_valid_content(),
+            (),
+        )
+        .expect("Content engine could not be created");
+        let content_engine = shared_content_engine.read().unwrap();
+
+        let template = "{{#each request.query-parameters}}{{@key}}: {{this}}\n{{/each}}";
+        let query_parameters = json!({
+            "hello": "world",
+            "goodbye": "moon",
+        });
+        let expected_outputs = [
+            String::from("hello: world\ngoodbye: moon\n"),
+            String::from("goodbye: moon\nhello: world\n"),
+        ];
+
+        let renderable = content_engine
+            .new_template(
+                template,
+                MediaType::from_media_range(mime::TEXT_PLAIN).unwrap(),
+            )
+            .expect("Template could not be parsed");
+        let rendered = renderable
+            .render(
+                content_engine.render_context(None, query_parameters),
+                &[mime::TEXT_PLAIN],
+            )
+            .expect(&format!("Template rendering failed for `{}`", template));
+        let actual_output = media_to_string(rendered);
+
+        assert!(
+            expected_outputs.contains(&actual_output),
+            "Template rendering for `{}` did not produce expected output (any of {:?}), got \"{}\"",
+            template,
+            expected_outputs,
+            actual_output,
+        );
+    }
 }
